module Spritely
	class SpritelyTool
		def initialize()
			@active = false
			@cursor_id = []
			@cursor_id << UI.create_cursor(File.join(PLUGINS_DIR, SPRITELY_DIR, "cursor_spritely-0.png"), 0, 31)
			@cursor_id << UI.create_cursor(File.join(PLUGINS_DIR, SPRITELY_DIR, "cursor_spritely-1.png"), 0, 31)
			
			reset
		end
		
		def reset()
			@state = 0
			@ipold = Sketchup::InputPoint.new
			@ip = Sketchup::InputPoint.new
			@vec = Geom::Vector3d.new
			@dragging = false
			@p0 = nil
		end
	
		def activate()
			@active = true
			reset
		end
		
		def onCancel(flag, view)
			view.invalidate
			reset
		end
		
		def deactivate(view)
			@active = false
			view.invalidate
		end
		
		def suspend(view)
			@active = false
			view.invalidate
		end
		
		def resume(view)
			@active = true
			view.invalidate
		end
		
		def active?()
			@active
		end
		
		def onMouseMove(flags, x, y, view)
			@ignoreIp = false
			snapLength = Sketchup.active_model.options['UnitsOptions']['LengthSnapLength']
			
			case @state
			when 0
				@ip.pick view, x, y
				@p0 = @ip.position
				if @ip != @ipold then
					view.invalidate if (@ip.display? or @ipold.display?)
					
					@ipold.copy! @ip
					
					view.tooltip = @ip.tooltip
				end
				
				Sketchup::set_status_text "Dimensions", SB_VCB_LABEL
			when 1
				@ip.pick view, x, y, @ipold
				@p1 = @ip.position
				@p1.z = @p0.z
				
				canSnap = (@ip.degrees_of_freedom == 3) or (@ip.face and (@ip.face.normal.z.abs == 1.0))
				
				@offset = @p1 - @p0
				
				size = @offset.x.abs > @offset.y.abs ? @offset.x.abs : @offset.y.abs
				@offset.x = @offset.x*size/@offset.x.abs
				@offset.y = @offset.y*size/@offset.y.abs
				
				@p1 = @p0 + @offset
				
				@p2 = @p0.clone
				@p3 = @p1.clone
				@p2[0], @p3[0] = @p3[0], @p2[0]
				
				Sketchup::set_status_text "Dimensions", SB_VCB_LABEL
				Sketchup::set_status_text Sketchup::format_length(@offset.x.abs) + ", " + Sketchup::format_length(@offset.y.abs), SB_VCB_VALUE
				
				view.invalidate
				
				if ((x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
					@dragging = true
				end
			end
		end
		
		def advanceState()
			case @state
			when 0
				if @ip.valid? then
					@state = 1
				end
			when 1
				if @ip.valid? then
					@ipold.copy! @ip
					@state = 2
					begin
						spritely
					ensure
						reset
						Sketchup.active_model.select_tool nil
					end
				end
			end
		end
		
		def onLButtonDown(flags, x, y, view)
			if @state == 0 then
				@xdown = x
				@ydown = y
			end
		
			advanceState
		end
		
		def onLButtonUp(flags, x, y, view)
			if (@state == 1) and @dragging then
				advanceState
			end
		end
		
		def onReturn(view)
			advanceState
		end
		
		def onUserText(text, view)
			Sketchup::set_status_text "", SB_VCB_VALUE
			
			case @state
			when 1
				result = text.match(/^\s*([^,]+)\s*(?:,\s*([^,]+)\s*)?$/)
				return unless result
				
				sizex = result[1].to_l
				sizey = result[2] ? result[2].to_l : sizex
				return unless sizex and sizey
				
				@offset = @p1 - @p0
				@offset.x = (@offset.x < 0) ? -sizex : sizex
				@offset.y = (@offset.y < 0) ? -sizey : sizey
					
				@p1 = @p0 + @offset
				
				@p2 = @p0.clone
				@p3 = @p1.clone
				@p2[0], @p3[0] = @p3[0], @p2[0]
				
				view.invalidate
				
				advanceState
			end
		end
		
		def draw(view)
			if @state >= 1 then
				view.drawing_color = 0x000000
				view.line_width = 1
				
				if (@state == 1) and (@offset.x.abs == @offset.y.abs) then
					view.line_stipple = "-"
					view.draw_line @p0, @p1
				end
				
				view.line_stipple = ""
				view.draw_polyline @p0, @p2, @p1, @p3, @p0
			end
			
			if @ip.display? and !@ignoreIp then
				@ip.draw view
			end
		end
		
		def getExtents()
			result = Geom::BoundingBox.new
			
			result.add @p0 if (@state >= 0) and @p0
			if (@state >= 1) and @p1 and @p2 and @p3 then
				result.add @p1
				result.add @p2
				result.add @p3
			end
			
			return result
		end
		
		def onSetCursor()
			UI.set_cursor(@cursor_id[@state])
		end
		
		def enableVCB?()
			return @state != 0
		end
		
		def onKeyDown(key, repeat, flags, view)
			if (key == CONSTRAIN_MODIFIER_KEY) && (repeat == 1) then
				@shift_down_time = Time.now
				
				if view.inference_locked? then
					view.lock_inference
				elsif @ip.valid? then
					view.lock_inference @ip
				end
			end
		end

		def onKeyUp(key, repeat, flags, view)
			if (key == CONSTRAIN_MODIFIER_KEY) && view.inference_locked? && ((Time.now - @shift_down_time) > 0.5) then
				view.lock_inference
			end
		end
		
		def spritely()
			Spritely.setRegion(Geom::Point3d.linear_combination(0.5, @p0, 0.5, @p1), (@p1.x - @p0.x).abs)
		end
    end
end
