module Spritely
	SPRITELY_DIR = "SPRITELY_Dir"
	PLUGINS_DIR = File.dirname(__FILE__)

	def Spritely.setupCamera(xangle, yangle, aspect_ratio)
		cam = Sketchup.active_model.active_view.camera
		width = @size
		height = width/aspect_ratio
		vec = Geom::Vector3d.new Math.sin(yangle)*Math.sin(xangle), -Math.cos(yangle)*Math.sin(xangle), Math.cos(xangle)
		vec.length = @size*10.0
		up = Geom::Vector3d.new -Math.sin(yangle)*Math.cos(xangle), Math.cos(yangle)*Math.cos(xangle), Math.sin(xangle)
		up.length = height*0.5
		
		target = @center + (Geom::Vector3d.new Math.sin(yangle)*width*0.5, -Math.cos(yangle)*width*0.5, 0) + up
		
		cam.perspective = false
		cam.height = height
		cam.aspect_ratio = aspect_ratio
		cam.set target+vec, target, up
	end
	
	def Spritely.asyncLoop(range, &proc)
		from = range.min
		to = range.max
		iteration = lambda do |index|
			last = (index > to)
			
			proc.call(index, last)
			
			if !last then
				UI.start_timer(0.02, false) { iteration.call(index.succ) }
			end
		end
		
		iteration.call(from)
		
		return nil
	end
	
	def Spritely.animateCamera(aspect_ratio)
		Spritely.asyncLoop(0...256) do |iz, last|
			Spritely.setupCamera(Math::PI/8.0, Math::PI*2.0*iz/256.0, aspect_ratio)
		end
	end
	
	def Spritely.setRegion(center, size)
		@center = center
		@size = size
	end
	
	def Spritely.render()
		begin
			Dir::mkdir(@path)
		rescue
		end
	
		cam = Sketchup.active_model.active_view.camera
		camstate = {
			:perspective => cam.perspective?,
			:aspect_ratio => cam.aspect_ratio,
			:fovheight => cam.perspective? ? cam.fov : cam.height,
			:eye => cam.eye,
			:target => cam.target,
			:up => cam.up
			}
		
		Spritely.asyncLoop(0...(@num)) do |i, last|
			if (!last) then
				Spritely.setupCamera(@angle*Math::PI/180.0, Math::PI*2.0*i/@num, @width*1.0/@height)
				Sketchup.active_model.active_view.write_image( {
					:filename => File.join(@path, @name + i.to_s + ".png"),
					:antialias => true,
					:transparent => true,
					:width => @width,
					:height => @height
				} )
			else
				cam.perspective		= camstate[:perspective]
				cam.aspect_ratio	= camstate[:aspect_ratio]
				if (cam.perspective?) then
					cam.fov			= camstate[:fovheight]
				else
					cam.height		= camstate[:fovheight]
				end
				cam.set camstate[:eye], camstate[:target], camstate[:up]
			end
		end
	end
	
	def Spritely.captureSprite()
		prompts = ["Sprite width (px)", "Sprite height (px)", "Viewing angle (degrees)", "Number of images", "Output path", "Name prefix"]
		defaults = [@width.to_s, @height.to_s, @angle.to_s, @num.to_s, @path, @name]
		input = UI.inputbox prompts, defaults, "Enter capture parameters."
		if (input != nil) then
			@width = input[0].to_i
			@height = input[1].to_i
			@angle = input[2].to_f
			@num = input[3].to_i
			@path = input[4]
			@name = input[5]
			
			render
		end
	end

	# Add menu items
	if not file_loaded? "spritely.rb" then
		require(File.join(SPRITELY_DIR, "spritely_tool.rb"))
	
		@width = 64
		@height = 128
		@angle = 22.5
		@num = 16
		@path = File.join(ENV['USERPROFILE'].gsub("\\", File::SEPARATOR), "Desktop", "Spritely")
		@name = "sprite"
		@center = Geom::Point3d.new 0, 0, 0
		@size = 100.0
		
		@myTool = SpritelyTool.new
		
		toolbar = UI::Toolbar.new "Spritely"
		
		cmd = UI::Command.new("Select Tile") { Sketchup.active_model.select_tool @myTool }
		cmd.small_icon = File.join(PLUGINS_DIR, SPRITELY_DIR, "spritely_small-0.png")
		cmd.large_icon = File.join(PLUGINS_DIR, SPRITELY_DIR, "spritely-0.png")
		cmd.tooltip = "Select Tile"
		cmd.status_bar_text = "Select the rectangle representing the tile beneath the sprite."
		cmd.set_validation_proc {
			if @myTool.active?
				MF_CHECKED
			else
				MF_ENABLED
			end
		}
		
		toolbar.add_item cmd

		cmd = UI::Command.new("Capture Sprite") { Spritely.captureSprite }
		cmd.small_icon = File.join(PLUGINS_DIR, SPRITELY_DIR, "spritely_small-1.png")
		cmd.large_icon = File.join(PLUGINS_DIR, SPRITELY_DIR, "spritely-1.png")
		cmd.tooltip = "Capture Sprite"
		cmd.status_bar_text = "Take pictures of the sprite from various angles."
		
		toolbar.add_item cmd
		
		toolbar.restore
	
		file_loaded("spritely.rb")
	end
end